#!/usr/bin/perl -w

# convert output of Windows BTc Sound Encoder by Roman Black
# takes MikroC .C table format and creates a progmem array
# plus some parameter #defines for avr-gcc
#
# Usage: create soundfile in BTc and export as 'PIC MikroC .C file'
# then run
#
# perl btc2avr.pl <MikroC.C >sound.h
#
# Michael Haberler April 2009


use Switch;

local($bits,$bytes,$alg,$method, $rate);

printf "// converted to AVR gcc syntax by btc2avr.pl\n//\n";
while (<>) {

    while ( <> ) {
	# copy everything upto array start, extracting
	# interesting parameters
	if  (! /void sound_data1.*$/) {
	    if ( /Sound encoded at\D+(\d+)\D+bits/ ) {
		$rate = $1;
	    }
	    if ( /Size\s+(\d+)\s+bits\s+\((\d+)\s+bytes/ ) {
		$bits = $1;
		$bytes = $2;
	    }
	    if ( /using\s+(\w+)\s+(\S+)\s+Algorithm/ ) {
		$alg = $1;
		$method = $2;
	    }
	    print $_;
	    next;
	} else {
	    
	    print "char sound[] PROGMEM = {\n";
	    goto DONE;
	}
    }
DONE:
    # we saw the start of the array.
    # extract bytes in one huge PROGMEM array.
    while (<>) {
	if ( /asm retlw\s+(\w+)\s+;/) {
	    printf "\t$1,\n";
	    next;
	}
    }
    printf "};\n";
    printf "#define BTC_ONE_BIT 0\n";
    printf "#define BTC_ONEPOINTFIVE_BIT 1\n";
    printf "#define BTC_UNKNOWN 2\n";

    printf "#define SOUND_BITS $bits\n";
    printf "#define SOUND_BYTES $bytes\n";
    printf "#define SOUND_ALGORITHM $alg\n";
    printf "#define SOUND_ALGORITHM_NAME \"$alg\"\n";

    switch ($method) {
	case "1bit" {printf "#define SOUND_ENCODING BTC_ONE_BIT\n" }
	case "1.5bit" {printf "#define SOUND_ENCODING BTC_ONEPOINTFIVE_BIT\n" }
	else {printf "#define SOUND_ENCODING BTC_UNKNOWN\n" }
    }
    printf "#define SOUND_RATE $rate\n";
}




